'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _mkdirp = require('mkdirp');

var _mkdirp2 = _interopRequireDefault(_mkdirp);

var _toposort = require('toposort');

var _toposort2 = _interopRequireDefault(_toposort);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _fs = require('fs');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

class AssetsPlugin {

  constructor({
    assetsFilename = 'webpack-assets.json',
    path
  }) {
    this.assetsFilename = assetsFilename;
    this.bundlePath = path;
  }

  apply(compiler) {
    var _this = this;

    compiler.plugin('after-emit', (() => {
      var _ref = _asyncToGenerator(function* (compilation, cb) {
        // now topologically sort assets and save them to bundle directory

        try {
          const publicPath = compilation.options.output.publicPath;

          const json = compilation.getStats().toJson();
          const initialChunks = json.chunks.filter(function (chunk) {
            return chunk.initial;
          });
          const chunks = {};
          const entryPoints = {};
          const modules = {};
          const bundles = {};

          json.modules.forEach(function (module) {
            const parts = module.identifier.split('!');
            const filePath = parts[parts.length - 1];
            modules[filePath] = module.chunks;
          });

          json.chunks.forEach(function (chunk) {
            bundles[chunk.id] = chunk.files.map(function (file) {
              return publicPath + file;
            });
          });

          initialChunks.forEach(function (chunk) {
            chunks[chunk.id] = chunk;
          });

          const edges = [];

          initialChunks.forEach(function (chunk) {
            if (chunk.parents) {
              // Add an edge for each parent (parent -> child)
              chunk.parents.forEach(function (parentId) {
                const parentChunk = chunks[parentId];
                // If the parent chunk does not exist (e.g. because of an excluded chunk)
                // we ignore that parent
                if (parentChunk) {
                  edges.push([parentChunk, chunk]);
                }
              });
            }
          });

          const sorted = _toposort2.default.array(initialChunks, edges);

          sorted.forEach(function (chunk) {
            let map;

            if (!entryPoints[chunk.names[0]]) {
              map = entryPoints[chunk.names[0]] = { css: [], js: [] };
            } else {
              map = entryPoints[chunk.names[0]];
            }

            const files = Array.isArray(chunk.files) ? chunk.files : [chunk.files];

            files.forEach(function (file) {
              const filePath = publicPath + file;

              if (/\.js$/.test(file)) {
                map.js.push(filePath);
              } else if (/\.css$/.test(file)) {
                map.css.push(filePath);
              }
            });
          });

          // create build directory just in case
          _mkdirp2.default.sync(_this.bundlePath);
          (0, _fs.writeFileSync)(_path2.default.resolve(_this.bundlePath, _this.assetsFilename), JSON.stringify({
            entryPoints,
            modules,
            bundles
          }));
        } catch (e) {
          cb(e);
        } finally {
          cb();
        }
      });

      return function (_x, _x2) {
        return _ref.apply(this, arguments);
      };
    })());
  }
}
exports.default = AssetsPlugin;
module.exports = exports['default'];