'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _progress = require('progress');

var _progress2 = _interopRequireDefault(_progress);

var _webpack = require('webpack');

var _webpack2 = _interopRequireDefault(_webpack);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

class Progress {

  constructor(compilers) {
    this.clientProgress = 0;
    this.serverProgress = 0;

    this.progress = () => {
      this.tick(this.clientProgress * 50 + this.serverProgress * 50);
    };

    this.start = () => {
      this.stop();
      this.progressBar = new _progress2.default('compiling [:bar] :percent :elapseds', {
        complete: '=',
        incomplete: ' ',
        width: 20,
        total: 100
      });
      this.interval = setInterval(this.progress, 250);
    };

    this.stop = () => {
      this.progressBar = null;
      clearInterval(this.interval);
      this.interval = null;
    };

    this.tick = progressInPercent => {
      if (this.progressBar != null) {
        this.progressBar.update(progressInPercent / 100);
        this.progressBar.render();
      }

      if (progressInPercent === 100) {
        this.stop();
      }
    };

    this.compilers = compilers;

    // apply progress plugins to compilers
    if (compilers.length < 2) {
      throw new Error('Expecting 2 compiler instances, client and server');
    }

    // client side progeess
    compilers[0].apply(new _webpack2.default.ProgressPlugin(progress => {
      const lastProgress = this.clientProgress;

      this.clientProgress = progress;

      if (lastProgress !== progress) {
        this.progress();
      }
    }));

    // server side progress
    compilers[1].apply(new _webpack2.default.ProgressPlugin(progress => {
      const lastProgress = this.serverProgress;

      this.serverProgress = progress;

      if (lastProgress !== progress) {
        this.progress();
      }
    }));
  }

}
exports.default = Progress;
module.exports = exports['default'];