'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _chalk = require('chalk');

var _chalk2 = _interopRequireDefault(_chalk);

var _mkdirp = require('mkdirp');

var _mkdirp2 = _interopRequireDefault(_mkdirp);

var _terminate = require('./terminate');

var _terminate2 = _interopRequireDefault(_terminate);

var _path = require('path');

var _fs = require('fs');

var _spawnServer = require('./spawnServer');

var _spawnServer2 = _interopRequireDefault(_spawnServer);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

class ServerProcessManager {

  constructor(port = 3000, workingDirectory) {
    this.spawnErrors = [];

    this.port = port;
    this.workingDirectory = workingDirectory;
  }

  spawnServer(serverFile) {
    var _this = this;

    return _asyncToGenerator(function* () {
      try {
        const spawnedServer = yield (0, _spawnServer2.default)(serverFile, _this.workingDirectory, _this.port);

        // pipe all logs
        spawnedServer.stdout.pipe(process.stdout);
        spawnedServer.stderr.pipe(process.stdout);

        return spawnedServer;
      } catch (e) {
        _this.spawnErrors.push(e);

        throw e;
      }
    })();
  }

  isRunning() {
    return this.currentServer != null;
  }

  killServer() {
    var _this2 = this;

    return _asyncToGenerator(function* () {
      const server = _this2.currentServer;

      return new Promise(function (resolve, reject) {
        if (server == null) {
          return resolve();
        }

        // remove all exit listeners so we won't log an error after spawn
        server.removeAllListeners('exit');

        (0, _terminate2.default)(server.pid, 30000).then(resolve).catch(reject);
      });
    })();
  }

  lastSpawnErrors() {
    return this.spawnErrors;
  }

  clearSpawnErrors() {
    this.spawnErrors = [];
  }

  manage(serverCode, bundleDirectory) {
    var _this3 = this;

    return _asyncToGenerator(function* () {
      // now save it to the bundle dir
      const serverCodeFile = (0, _path.resolve)(bundleDirectory, `./server.${Date.now()}.js`);

      try {
        // reset spawn errors
        _this3.spawnErrors = [];
        _mkdirp2.default.sync(bundleDirectory);
        (0, _fs.writeFileSync)(serverCodeFile, serverCode);

        // now kill previous server
        yield _this3.killServer();

        _this3.currentServer = null;

        const serverProc = yield _this3.spawnServer(serverCodeFile);

        serverProc.once('exit', function (code) {
          console.log(_chalk2.default.red(`Server unexpectedly terminated with the code ${code}`));
        });

        // remove previous server code file
        if (_this3.currentServerCodeFile) {
          (0, _fs.unlinkSync)(_this3.currentServerCodeFile);
          _this3.currentServerCodeFile = null;
        }

        _this3.currentServerCodeFile = serverCodeFile;
        _this3.currentServer = serverProc;
      } catch (e) {
        // remove new server code
        if ((0, _fs.existsSync)(serverCodeFile)) {
          (0, _fs.unlinkSync)(serverCodeFile);
        }

        // respawn previous instance
        if (_this3.currentServerCodeFile) {
          const serverProc = yield _this3.spawnServer(_this3.currentServerCodeFile);

          _this3.currentServer = serverProc;
        } else {
          throw e;
        }
      }
    })();
  }

  close() {
    var _this4 = this;

    return _asyncToGenerator(function* () {
      yield _this4.killServer();

      _this4.currentServer = null;

      // remove server code
      if (_this4.currentServerCodeFile) {
        (0, _fs.unlinkSync)(_this4.currentServerCodeFile);
        _this4.currentServerCodeFile = null;
      }
    })();
  }
}
exports.default = ServerProcessManager;
module.exports = exports['default'];