#!/usr/bin/env node
'use strict';

let build = (() => {
  var _ref = _asyncToGenerator(function* (dir, sourceDir, { outputStats }) {
    const useBabili = !!parseInt(process.env.SPUST_USE_BABILI || '0', 10);
    let config = yield (0, _configure2.default)({ workDir: dir, srcDir: sourceDir, env: 'production', useBabili });

    // override configuration using own configure
    const configPath = (0, _path.resolve)(workDir, './spust.config.js');

    try {
      // dost spust.config.js file exist?
      yield (0, _fs.stat)(configPath);

      delete require.cache[require.resolve(configPath)];
      let customConfigure = require(configPath);

      config = customConfigure(config, { workDir: dir, srcDir: sourceDir, env: 'production' });
    } catch (e) {
      if (e.code !== 'ENOENT') {
        throw e;
      }
    }

    yield new Promise(function (resolve, reject) {
      return (0, _rimraf2.default)((0, _path.resolve)(workDir, 'bundle'), function (err) {
        if (err) {
          reject(err);
        } else {
          resolve();
        }
      });
    });

    return yield new Promise(function (resolve, reject) {
      const compiler = (0, _webpack2.default)([config.client, config.server]);
      const progress = new _Progress2.default(compiler.compilers);
      progress.start();

      compiler.run((() => {
        var _ref2 = _asyncToGenerator(function* (err, stats) {
          progress.stop();

          if (isInteractive) {
            (0, _clearConsole2.default)();
          }

          if (err) {
            reject(err.stack || err);
            return;
          }

          const [clientStats, serverStats] = stats.stats;

          const clientMessages = (0, _formatWebpackMessages2.default)(clientStats.toJson({}, true));
          const serverMessages = (0, _formatWebpackMessages2.default)(serverStats.toJson({}, true));

          if (clientMessages.errors.length || serverMessages.errors.length) {
            console.log(_chalk2.default.red('Failed to compile.\n'));
            console.log([...clientMessages.errors, ...serverMessages.errors].join('\n'));
            reject();
            return;
          }

          if (clientMessages.warnings.length || serverMessages.warnings.length) {
            console.log(_chalk2.default.yellow('Compiled with warnings.\n'));
            console.log([...clientMessages.warnings, ...serverMessages.warnings].join('\n\n'));
          } else {
            console.log(_chalk2.default.green('Successfully built. See bundle directory.'));
          }

          // write stats.json if is enabled
          if (outputStats) {
            const clientStatsFile = (0, _path.resolve)(dir, 'client.stats.json');
            const serverStatsFile = (0, _path.resolve)(dir, 'server.stats.json');

            yield Promise.all([(0, _fs.writeFile)(clientStatsFile, JSON.stringify(clientStats.toJson('verbose'), null, 2)), (0, _fs.writeFile)(serverStatsFile, JSON.stringify(serverStats.toJson('verbose'), null, 2))]);

            console.log(_chalk2.default.cyan('\nYou can analyze your webpack bundles. See following json files:\n'));
            console.log(` client bundle stats: ${clientStatsFile}`);
            console.log(` server bundle stats: ${serverStatsFile}`);
            console.log();
            console.log(_chalk2.default.cyan('Visit https://webpack.github.io/analyse/ and use one of the files mentioned above.'));
          }

          resolve();
        });

        return function (_x4, _x5) {
          return _ref2.apply(this, arguments);
        };
      })());
    });
  });

  return function build(_x, _x2, _x3) {
    return _ref.apply(this, arguments);
  };
})();

var _path = require('path');

var _fs = require('mz/fs');

var _clearConsole = require('react-dev-utils/clearConsole');

var _clearConsole2 = _interopRequireDefault(_clearConsole);

var _formatWebpackMessages = require('react-dev-utils/formatWebpackMessages');

var _formatWebpackMessages2 = _interopRequireDefault(_formatWebpackMessages);

var _chalk = require('chalk');

var _chalk2 = _interopRequireDefault(_chalk);

var _rimraf = require('rimraf');

var _rimraf2 = _interopRequireDefault(_rimraf);

var _Progress = require('./Progress');

var _Progress2 = _interopRequireDefault(_Progress);

var _webpack = require('webpack');

var _webpack2 = _interopRequireDefault(_webpack);

var _configure = require('./configure');

var _configure2 = _interopRequireDefault(_configure);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

/* eslint-disable import/first */

process.env.BABEL_ENV = 'production';
process.env.NODE_ENV = 'production';

// crash on unhandledRejection
process.on('unhandledRejection', err => {
  throw err;
});

const isInteractive = process.stdout.isTTY;
const srcDir = process.argv.slice(3).find(arg => !arg.startsWith('--')) || 'src';
const workDir = process.argv[2];
const args = new Set(process.argv);
const outputStats = args.has('--stats');

if (workDir == null) {
  throw new Error('Please provide workDir path as the first argument');
}

build(workDir, srcDir, { outputStats }).then(() => {
  process.exit(0);
}).catch(e => {
  if (e) {
    console.log(e);
  }

  process.exit(1);
});