'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

let configure = (() => {
  var _ref = _asyncToGenerator(function* ({
    env,
    serverManager,
    srcDir,
    useBabili,
    workDir
  }) {
    const isDev = env === 'development';
    const appDir = (0, _path.resolve)(workDir, srcDir);
    const clientBundlePath = (0, _path.resolve)(workDir, './bundle/client');
    const serverBundlePath = (0, _path.resolve)(workDir, './bundle/server');

    const variables = Object.assign({
      // provide full path to webpack-assets.json so you can easily load it server side
      ASSETS_JSON_PATH: (0, _path.resolve)(serverBundlePath, 'webpack-assets.json'),
      ASSETS_PATH: clientBundlePath,
      CLIENT_BUNDLE_PATH: clientBundlePath,
      SERVER_BUNDLE_PATH: serverBundlePath,
      NODE_ENV: env
    }, (yield (0, _helpers.loadEnvVariables)(workDir)), isDev ? { PORT: process.env.PORT } : {});

    const envVariables = Object.keys(variables).reduce(function (res, key) {
      return Object.assign({}, res, { [`process.env.${key}`]: JSON.stringify(variables[key]) });
    }, {});

    return {
      client: {
        bail: !isDev,
        devtool: isDev ? 'cheap-module-source-map' : 'source-map',
        entry: [isDev ? require.resolve('react-dev-utils/webpackHotDevClient') : null, require.resolve('./polyfills/client'), isDev ? require.resolve('react-error-overlay') : null, (0, _path.resolve)(appDir, './client/index.js')].filter(Boolean),
        output: {
          filename: isDev ? 'static/js/[name].js' : 'static/js/[name].[chunkhash:8].js',
          chunkFilename: 'static/js/[name].[chunkhash:8].js',
          libraryTarget: 'var',
          path: clientBundlePath,
          pathinfo: isDev,
          publicPath: '/',
          devtoolModuleFilenameTemplate: isDev ? function (info) {
            return (0, _path.resolve)(info.absoluteResourcePath);
          } : function (info) {
            return (0, _path.relative)(appDir, info.absoluteResourcePath);
          }
        },
        resolve: {
          extensions: ['.js', '.json', '.jsx'],
          modules: ['node_modules', (0, _path.resolve)(workDir, './node_modules')],
          alias: {
            'babel-runtime': (0, _path.dirname)(require.resolve('babel-runtime/package.json'))
          }
        },
        module: {
          strictExportPresence: true,
          rules: (0, _helpers.createWebpackLoaders)({ appDir, isDev, isServer: false, useBabili }, envVariables)
        },
        node: {
          dgram: 'empty',
          fs: 'empty',
          net: 'empty',
          tls: 'empty'
        },
        performance: isDev ? false : {
          hints: 'warning'
        },
        plugins: (0, _helpers.createWebpackPlugins)({ isDev, isServer: false, clientBundlePath, serverBundlePath, serverManager, useBabili }, envVariables),
        target: 'web'
      },
      server: {
        bail: !isDev,
        devtool: isDev ? 'inline-source-map' : 'source-map',
        entry: [isDev ? require.resolve('source-map-support/register') : null, require.resolve('./polyfills/server'), require.resolve('./helpers/provideVariables'), (0, _path.resolve)(appDir, './server/index.js')].filter(Boolean),
        externals: [(0, _webpackNodeExternals2.default)({
          whitelist: [/\.(eot|woff|woff2|ttf|otf)$/, /\.(svg|png|jpg|jpeg|gif|ico)$/, /\.(mp4|mp3|ogg|swf|webp)$/, /\.(css|scss|sass|sss|less)$/, /react-loadable/, /is-webpack-bundle/, /webpack-require-weak/]
        })],
        output: {
          filename: 'server.js',
          libraryTarget: 'commonjs2',
          path: serverBundlePath,
          publicPath: '/',
          devtoolModuleFilenameTemplate: function (info) {
            return (0, _path.relative)(appDir, info.absoluteResourcePath);
          }
        },
        resolve: {
          extensions: ['.js', '.json', '.jsx'],
          modules: ['node_modules', (0, _path.resolve)(workDir, './node_modules')]
        },
        module: {
          strictExportPresence: true,
          rules: (0, _helpers.createWebpackLoaders)({ appDir, isDev, isServer: true, useBabili }, envVariables)
        },
        node: {
          __dirname: true,
          __filename: true
        },
        plugins: (0, _helpers.createWebpackPlugins)({ isDev, isServer: true, clientBundlePath, serverBundlePath, serverManager, useBabili }, envVariables),
        performance: false,
        target: 'node'
      }
    };
  });

  return function configure(_x) {
    return _ref.apply(this, arguments);
  };
})();

var _path = require('path');

var _webpackNodeExternals = require('webpack-node-externals');

var _webpackNodeExternals2 = _interopRequireDefault(_webpackNodeExternals);

var _helpers = require('./helpers');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

exports.default = configure;
module.exports = exports['default'];