'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = createWebpackPlugins;

var _AssetsPlugin = require('../AssetsPlugin');

var _AssetsPlugin2 = _interopRequireDefault(_AssetsPlugin);

var _babiliWebpackPlugin = require('babili-webpack-plugin');

var _babiliWebpackPlugin2 = _interopRequireDefault(_babiliWebpackPlugin);

var _caseSensitivePathsWebpackPlugin = require('case-sensitive-paths-webpack-plugin');

var _caseSensitivePathsWebpackPlugin2 = _interopRequireDefault(_caseSensitivePathsWebpackPlugin);

var _compressionWebpackPlugin = require('compression-webpack-plugin');

var _compressionWebpackPlugin2 = _interopRequireDefault(_compressionWebpackPlugin);

var _extractTextWebpackPlugin = require('extract-text-webpack-plugin');

var _extractTextWebpackPlugin2 = _interopRequireDefault(_extractTextWebpackPlugin);

var _WatchMissingNodeModulesPlugin = require('react-dev-utils/WatchMissingNodeModulesPlugin');

var _WatchMissingNodeModulesPlugin2 = _interopRequireDefault(_WatchMissingNodeModulesPlugin);

var _webpack = require('webpack');

var _webpack2 = _interopRequireDefault(_webpack);

var _ServerListenerPlugin = require('../ServerListenerPlugin');

var _ServerListenerPlugin2 = _interopRequireDefault(_ServerListenerPlugin);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// detect offline plugin
let OfflinePlugin;

try {
  OfflinePlugin = require('offline-plugin');
} catch (e) {}

function createWebpackPlugins({
  isDev,
  isServer,
  serverBundlePath,
  serverManager,
  useBabili
}, envVariables) {
  let serverListenerPlugin;

  if (isDev && isServer) {
    if (serverManager == null) {
      throw new Error('Server manager must be provided during development');
    }

    serverListenerPlugin = new _ServerListenerPlugin2.default(serverManager);
  }

  return [
  // add module names to factory functions so they appear in browser profiler
  !isServer && isDev ? new _webpack2.default.NamedModulesPlugin() : null,

  // define variables client side and server side
  new _webpack2.default.DefinePlugin(Object.assign({}, envVariables, {
    'process.env.IS_SERVER': JSON.stringify(isServer),
    'process.env.IS_CLIENT': JSON.stringify(!isServer)
  })),

  // hot module replacement, only client side and dev
  isDev && !isServer ? new _webpack2.default.HotModuleReplacementPlugin() : null,

  // case sensitive paths plugin, only dev, both sides
  isDev ? new _caseSensitivePathsWebpackPlugin2.default() : null,

  // watch missing node modules plugin, only dev, both sides
  isDev ? new _WatchMissingNodeModulesPlugin2.default() : null,

  // make everything that is pointed from 2 and more places a chunk
  !isDev && !isServer ? new _webpack2.default.optimize.CommonsChunkPlugin({
    async: true,
    children: true,
    minChunks: (module, count) => count >= 2
  }) : null,

  // uglify js, only client side and prod mode
  !isDev && !isServer && !useBabili ? new _webpack2.default.optimize.UglifyJsPlugin({
    compress: {
      warnings: false,
      comparisons: false
    },
    output: {
      comments: false
    },
    sourceMap: true
  }) : null,

  // babili, only client side and prod mode
  !isDev && !isServer && useBabili ? new _babiliWebpackPlugin2.default({}, {
    babel: require('babel-core'),
    babili: require.resolve('babel-preset-babili'),
    comments: false
  }) : null,

  // ignore locales imported by momentjs, both sides, requires you to require them explicitly
  new _webpack2.default.IgnorePlugin(/^\.\/locale$/, /moment$/),

  // extract css, only client side and prod
  !isDev && !isServer ? new _extractTextWebpackPlugin2.default({
    filename: 'static/css/[name].[hash:8].css',
    allChunks: true
  }) : null,

  // compress to gz, only client side assets and prod
  !isDev && !isServer ? new _compressionWebpackPlugin2.default({
    asset: '[path].gz',
    algorithm: 'gzip'
  }) : null,

  // offline plugin , only client side assets and prod mode
  !isDev && !isServer && OfflinePlugin != null ? new OfflinePlugin({
    AppCache: false,
    autoUpdate: true,
    caches: {
      main: ['**/vendor.*.{css,js}', '**/main.*.{css,js}'],
      additional: [':rest:']
      // main: ['**/vendor-*.{css,js}', '**/main-*.{css,js}'],
      // additional: ['**/*.{css,js,jpg,jpeg,png,ogg,svg,woff,woff2,ttf}'],
      // additional: [':rest:'],
    },
    excludes: ['**/*.map', '**/*.map.gz'],
    publicPath: '/',
    safeToUseOptionalCaches: true,
    ServiceWorker: {
      // emit events so we can react to them in client/index.js
      events: true,
      output: './static/js/sw.js',
      publicPath: '/static/js/sw.js'
    }
  }) : null,

  // limit chunks to only one file on server side (useful for react-loadable)
  isServer ? new _webpack2.default.optimize.LimitChunkCountPlugin({ maxChunks: 1 }) : null,

  // assets plugin, only client side and prod mode
  !isServer ? new _AssetsPlugin2.default({ path: serverBundlePath }) : null,

  // server listener plugin to manage exported server instance
  serverListenerPlugin].filter(Boolean);
}
module.exports = exports['default'];