'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = portPid;

var _selectiveWhitespace = require('selective-whitespace');

var _selectiveWhitespace2 = _interopRequireDefault(_selectiveWhitespace);

var _netstats = require('netstats');

var _netstats2 = _interopRequireDefault(_netstats);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const platform = process.platform;
const isWindows = platform === 'win32';

function processResult(rows) {
  const pidIndex = 1;
  const items = isWindows ? rows : rows.slice(1);
  const result = items.reduce((res, row) => {
    const values = (0, _selectiveWhitespace2.default)(row).split(' ');
    const pid = parseInt(isWindows ? values.pop() : values[pidIndex], 10);

    if (pid === '' || pid === 0 || isNaN(pid)) {
      return res;
    }

    if (values.includes('TCP')) {
      res.tcp.add(pid);
    }

    if (values.includes('UDP')) {
      res.udp.add(pid);
    }

    return res;
  }, { tcp: new Set(), udp: new Set() });

  return Object.assign({}, result, {
    all: new Set([...result.tcp, ...result.udp])
  });
}

function portPid(port) {
  return new Promise((resolve, reject) => {
    (0, _netstats2.default)(port).then(processResult).then(resolve).catch(() => resolve({
      all: new Set(),
      tcp: new Set(),
      udp: new Set()
    }));
  });
}

// Example output to parse:

// [MAC] $ lsof -i :8017
/*
 COMMAND   PID    USER   FD   TYPE             DEVICE SIZE/OFF NODE NAME
 nc      20661 michael    3u  IPv4 0x3b190d9d07c2c3db      0t0  TCP *:8017 (LISTEN)
 nc      21145 michael    3u  IPv4 0x3b190d9d054773db      0t0  TCP *:8017 (LISTEN)
 Python  21221 michael    3u  IPv4 0x3b190d9ceb8dfd7b      0t0  UDP localhost:8017
 */

// [WIN] $ netstat.exe -a -n -o | findstr :9000

/*
 TCP    0.0.0.0:9000           0.0.0.0:0              LISTENING       5220
 TCP    127.0.0.1:9000         127.0.0.1:62376        ESTABLISHED     5220
 TCP    127.0.0.1:9000         127.0.0.1:62379        ESTABLISHED     5220
 TCP    127.0.0.1:62288        127.0.0.1:9000         TIME_WAIT       0
 TCP    127.0.0.1:62299        127.0.0.1:9000         TIME_WAIT       0
 TCP    127.0.0.1:62376        127.0.0.1:9000         ESTABLISHED     7604
 TCP    127.0.0.1:62378        127.0.0.1:9000         ESTABLISHED     7604
 UDP    127.0.0.1:9000         *:*                                    1240
 */

module.exports = exports['default'];