'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.assets = assets;
exports.createAsyncRequireLoader = createAsyncRequireLoader;

var _fs = require('fs');

// this file expose helpers that you can use in your project
function assets() {
  // see helpers/providerVariables.js and configure.js server section's entry point
  const assetsJsonPath = process.env.__ASSETS_JSON_PATH || '';
  const loadedAssets = JSON.parse((0, _fs.readFileSync)(assetsJsonPath, { encoding: 'utf8' }));

  return Object.keys(loadedAssets.entryPoints).reduce((result, entryName) => {
    const { css, js } = loadedAssets.entryPoints[entryName];

    return {
      css: [...result.css, ...(css == null ? [] : Array.isArray(css) ? css : [css])],
      js: [...result.js, ...(js == null ? [] : Array.isArray(js) ? js : [js])]
    };
  }, { css: [], js: [] });
}

function createAsyncRequireLoader() {
  // see helpers/providerVariables.js and configure.js server section's entry point
  const assetsJsonPath = process.env.__ASSETS_JSON_PATH || '';
  const loadedAssets = JSON.parse((0, _fs.readFileSync)(assetsJsonPath, { encoding: 'utf8' }));

  return {
    getScripts(requiredFiles) {
      return requiredFiles.reduce((result, requiredFile) => {
        const file = requiredFile + '.js';
        const indexFile = requiredFile + '/index.js';

        const bundles = loadedAssets.modules[file] || loadedAssets.modules[indexFile];

        if (bundles == null) {
          return result;
        }

        const scripts = bundles.reduce((processed, bundle) => {
          const files = loadedAssets.bundles[bundle];

          if (files == null) {
            return processed;
          }

          // allow only js files
          return [...files.filter(fileName => fileName.endsWith('js')).reverse(), ...processed];
        }, []);

        return [...scripts, ...result];
      }, []);
    }
  };
}